#include<iostream>
#include<vector>
#include<cmath>
#include<limits>

template<typename T>
struct AD {
  T v=0,d=0;
  AD(const T& v) : v(v), d(0) {}
  void diff_wrt() { d=1; }
};

template<typename T>
AD<T> sin(const AD<T>& x) {
  AD<T> y(0);
  y.v=sin(x.v);
  y.d=cos(x.v)*x.d;
  return y;
}

template<typename T>
AD<T> cos(const AD<T>& x) {
  AD<T> y(0);
  y.v=cos(x.v);
  y.d=-sin(x.v)*x.d;
  return y;
}

template<typename T>
AD<T> operator-(const AD<T>& x) {
  AD<T> y(0);
  y.v=-x.v;
  y.d=-x.d;
  return y;
}

template<typename T>
AD<T> operator+(const AD<T>& x1, const AD<T>& x2) {
  AD<T> y(0);
  y.v=x1.v+x2.v;
  y.d=x1.d+x2.d;
  return y;
}

template<typename T>
AD<T> operator*(const AD<T>& x1, const AD<T>& x2) {
  AD<T> y(0);
  y.v=x1.v*x2.v;
  y.d=x1.d*x2.v+x1.v*x2.d;
  return y;
}

template<typename T>
std::ostream& operator<<(std::ostream& s, const AD<T>& x) {
  s << "(" << x.v << "," << x.d << ")";
  return s;
}

template<typename T>
T f(T x) {
  return sin(x);
}

template<typename T>
T dfdx(const T& x) {
  AD<T> x_ad(x); x_ad.diff_wrt();
  return f(x_ad).d;
}

template<typename T>
T ddfdxx(const T& x) {
  AD<T> x_ad(x); x_ad.diff_wrt();
  return dfdx(x_ad).d;
}

template<typename T>
T dfdx_fd(const T& x) {
  const T h=(x==0) ? sqrt(sqrt(std::numeric_limits<T>::epsilon()))
    : sqrt(sqrt(std::numeric_limits<T>::epsilon()))*fabs(x);
  return (f(x+h)-f(x-h))/(2*h);
}

template<typename T>
T ddfdxx_fd(const T& x) {
  const T h=(x==0) ? sqrt(sqrt(std::numeric_limits<T>::epsilon()))
    : sqrt(sqrt(std::numeric_limits<T>::epsilon()))*fabs(x);
  return (dfdx_fd(x+h)-dfdx_fd(x-h))/(2*h);
}

int main() {
  using T=double;
  T x=1;
  std::cout << f(x) << std::endl;
  std::cout << dfdx(x) << std::endl;
  std::cout << dfdx_fd(x) << std::endl;
  std::cout << ddfdxx(x) << std::endl;
  std::cout << ddfdxx_fd(x) << std::endl;
  return 0;
}
