#include<iostream>
#include<vector>
#include<cmath>

template<typename T>
AD<T> sin(const AD<T>& x) {
  AD<T> y;
  y.v=sin(x.v);
  y.d=cos(x.v)*x.d;
  return y;
}

template<typename T>
AD<T> cos(const AD<T>& x) {
  AD<T> y;
  y.v=cos(x.v);
  y.d=-sin(x.v)*x.d;
  return y;
}

template<typename T>
AD<T> pow(const AD<T>& x, int n) {
  AD<T> y;
  y.v=pow(x.v,n);
  y.d=n*pow(x.v,n-1)*x.d;
  return y;
}

template<typename T>
AD<T> fabs(const AD<T>& x) {
  AD<T> y;
  y.v=fabs(x.v);
  y.d=x.d;
  if (x.v<0) y.d=-x.d;
  return y;
}

template<typename T>
AD<T> sqrt(const AD<T>& x) {
  AD<T> y;
  y.v=sqrt(x.v);
  y.d=1./(2*y.v)*x.d;
  return y;
}

template<typename T>
AD<T> operator-(const AD<T>& x) {
  AD<T> y;
  y.v=-x.v;
  y.d=-x.d;
  return y;
}

template<typename T>
AD<T> operator+(const AD<T>& x1, const AD<T>& x2) {
  AD<T> y;
  y.v=x1.v+x2.v;
  y.d=x1.d+x2.d;
  return y;
}

template<typename T>
AD<T> operator-(const AD<T>& x1, const AD<T>& x2) {
  AD<T> y;
  y.v=x1.v-x2.v;
  y.d=x1.d-x2.d;
  return y;
}

template<typename T>
AD<T> operator-(const AD<T>& x1, const T& x2) {
  AD<T> y;
  y.v=x1.v-x2;
  y.d=x1.d;
  return y;
}

template<typename T>
AD<T> operator*(const AD<T>& x1, const AD<T>& x2) {
  AD<T> y;
  y.v=x1.v*x2.v;
  y.d=x1.d*x2.v+x1.v*x2.d;
  return y;
}

template<typename T>
AD<T> operator/(const AD<T>& x1, const AD<T>& x2) {
  AD<T> y;
  y.v=x1.v/x2.v;
  y.d=x1.d/x2.v-x1.v*x2.d/(x2.v*x2.v);
  return y;
}

template<typename T>
bool operator>(const AD<T>& x1, const AD<T>& x2) {
  return (x1.v>x2.v);
}

template<typename T>
bool operator<=(const AD<T>& x1, const AD<T>& x2) {
  return (x1.v<=x2.v);
}

template<typename T>
bool operator<(const AD<T>& x1, const AD<T>& x2) {
  return (x1.v<x2.v);
}

template<typename T>
bool operator!=(const AD<T>& x1, const AD<T>& x2) {
  return (x1.v!=x2.v);
}

template<typename T>
std::ostream& operator<<(std::ostream& s, const AD<T>& x) {
  s << x.v;
  return s;
}

template<typename T>
std::istream& operator>>(std::istream& s, AD<T>& x) {
  s >> x.v;
  return s;
}
